#!/usr/bin/fades

import argparse
import pathlib
import re
import textwrap
from collections import defaultdict

import requests  # fades

APIKEY = 'f405a1983ffded918cd46f2b3ff53c08'

BASE_URL = 'https://api.themoviedb.org/3'

AVOID_CHARS_TT = str.maketrans("", "", "'\"!\\/`")


class InternalError(Exception):
    """Internal error when parsing filenames."""


def hit_tmdb(endpoint, params=None):
    """Hit the DB service."""
    if params is None:
        params = {}
    params['api_key'] = APIKEY
    response = requests.get(BASE_URL + endpoint, params=params)
    content = response.json()
    if not content.get("success", True):
        msg = content["status_message"]
        print("ERROR on TMDB:", msg)
        print("    url:", endpoint)
        exit(1)
    return content


def search_db(series_name):
    """Search the DB for the given series name."""
    endpoint = '/search/tv'
    params = {
        'query': series_name,
        'include_adult': True,
    }
    response = hit_tmdb(endpoint, params)
    found = []
    for item in response.get('results', []):
        found.append((item['original_name'], item['id'], item['overview']))

    if not found:
        raise InternalError("Nothing found in the DB (searched {!r})".format(series_name))

    print("Found:")
    for idx, (name, tv_id, overview) in enumerate(found):
        print("    {:d}. {} [{}]".format(idx, name, tv_id))
        wrapped = textwrap.wrap(overview)
        print(textwrap.indent("\n".join(wrapped), " " * 8))
    if len(found) == 1:
        return name, tv_id

    while True:
        opt = input("Choose: ")
        try:
            name, tv_id, _ = found[int(opt)]
        except (ValueError, IndexError):
            pass
        else:
            break

    return name, tv_id


def get_episodes(tv_id, season):
    """Get the info of the indicated season for the series."""
    endpoint = '/tv/{}/season/{}'.format(tv_id, season)
    response = hit_tmdb(endpoint)

    result = []
    for item in response['episodes']:
        name = item['name'].translate(AVOID_CHARS_TT)
        result.append((item['episode_number'], name))

    return result


def get_series_name(basedirpath):
    """Get the name of the series from the files in the directory."""
    found = []
    for filepath in basedirpath.iterdir():
        if m := re.match(r"(.*)[\. \[][sS](\d\d)[eE](\d\d).*", filepath.name):
            found.append((m.groups(), filepath.name, filepath.suffix[1:]))
        elif m := re.match(r"(.*) - (\d)x(\d\d).*", filepath.name):
            found.append((m.groups(), filepath.name, filepath.suffix[1:]))
        else:
            raise InternalError("Series name not understood: {!r}".format(filepath))

    season = None
    name = None
    episodes = defaultdict(list)
    for (f_name, f_season, f_episode), filename, f_ext in found:
        # replace dots by spaces, and un-camelcase it if needed
        if '.' in f_name:
            f_name = f_name.replace('.', ' ')
        if ' ' not in f_name and not f_name.islower():
            f_name = ' '.join(re.findall(r'[A-Z](?:[a-z]+|[A-Z]*(?=[A-Z]|$))', f_name))
        f_name = f_name.lower().strip()

        # store the name first time, check it's the same the next ones
        if name is None:
            name = f_name
        elif name != f_name:
            raise InternalError("more than one series name: {!r} and {!r}".format(name, f_name))

        f_season = int(f_season)
        if season is None:
            season = f_season
        elif season != f_season:
            raise InternalError(
                "more than one series season: {!r} and {!r}".format(season, f_season))

        episodes[int(f_episode)].append((filename, f_ext))

    return name, season, episodes


def main(basedirpath, overriden_name=None, list_remote=None):
    """Main entry point."""
    current_name, season, current_episodes = get_series_name(basedirpath)
    print("Found in disk: {!r} (season={}, {} episodes)".format(
        current_name, season, len(current_episodes)))

    if overriden_name:
        print("Overriding name {!r} -> {!r}".format(current_name, overriden_name))
        current_name = overriden_name

    real_name, tv_id = search_db(current_name)
    print("Found TMDB id: {!r}".format(tv_id))

    db_episodes = get_episodes(tv_id, season)
    if list_remote:
        for ep in db_episodes:
            print(ep)
        return

    if len(db_episodes) != len(current_episodes):
        err = f"Different episodes quantity ({len(db_episodes)} in db): {db_episodes}"
        raise InternalError(err)

    moves = []
    for ep_num, ep_name in db_episodes:
        for current_name, ep_ext in current_episodes[ep_num]:
            new_name = "{} [s{:02d}e{:02d}] {}.{}".format(
                real_name, season, ep_num, ep_name, ep_ext)
            current_path = basedirpath / current_name
            new_path = basedirpath / new_name
            moves.append((current_path, new_path))

    # some security verifications
    current_total_files = len(list(basedirpath.iterdir()))
    if len(moves) != current_total_files:
        raise InternalError("Bad quantity of move commands!")
    if len({x[1] for x in moves}) != current_total_files:
        raise InternalError("Destination names overlap themselves!")
    for current_path, new_path in moves:
        if current_path.suffix != new_path.suffix:
            raise InternalError("Suffix mismatch!")

    # execute
    for current_path, new_path in moves:
        print("Renaming:", repr(current_path), repr(new_path))
        current_path.rename(new_path)
    print("Done.")


if __name__ == "__main__":
    parser = argparse.ArgumentParser()
    parser.add_argument("directory", type=pathlib.Path, help="The directory to find the files")
    parser.add_argument(
        "--name", help="The name of the series (will not be deducted from file names)")
    parser.add_argument(
        "--list", action="store_true", help="List remote content (do not change local)")
    args = parser.parse_args()
    try:
        main(args.directory, args.name, args.list)
    except InternalError as err:
        print("ERROR:", err)
